<?php

namespace Drupal\group_member_mgmt\GroupMembership;

use Drupal\group\Entity\GroupTypeInterface;
use Drupal\group\Plugin\GroupMembershipPermissionProvider as GroupMembershipPermissionProviderBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class GroupMembershipPermissionProvider.
 */
class GroupMembershipPermissionProvider extends GroupMembershipPermissionProviderBase {

  /**
   * The current route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container, $plugin_id, array $definition) {
    $plugin = parent::createInstance($container, $plugin_id, $definition);
    $plugin->routeMatch = $container->get('current_route_match');
    return $plugin;
  }

  /**
   * {@inheritdoc}
   */
  public function buildPermissions() {
    $permissions = parent::buildPermissions();
    // Not great, but we've got to obtain the group type somehow.
    if (!$groupType = $this->getGroupTypeFromRoute()) {
      return $permissions;
    }

    foreach ($this->getGroupMemberRoles($groupType) as $groupRole) {
      $createPerm = 'create group_membership content group-role:' . $groupRole->id();
      $viewPerm = 'view group_membership content group-role:' . $groupRole->id();
      $updatePerm = 'update group_membership content group-role:' . $groupRole->id();
      $deletePerm = 'delete group_membership content group-role:' . $groupRole->id();

      $permissions[$createPerm] = [
        'title' => 'Create memberships (role %role)',
        'title_args' => ['%role' => $groupRole->label()],
      ];

      $permissions[$viewPerm] = [
        'title' => 'View individual group members (role %role)',
        'title_args' => ['%role' => $groupRole->label()],
      ];

      $permissions[$updatePerm] = [
        'title' => 'Update memberships (role %role)',
        'title_args' => ['%role' => $groupRole->label()],
      ];

      $permissions[$deletePerm] = [
        'title' => 'Remove memberships (role %role)',
        'title_args' => ['%role' => $groupRole->label()],
      ];

      // Adjustments for standard "member" only.
      if ($groupRole->id() == $groupType->id() . '-member') {
        $permissions[$createPerm]['title'] = 'Create memberships';
        $permissions[$createPerm]['description'] = 'Create memberships (no group roles) for existing users. Needed for creating memberships, use additional permissions to allow for selection of additional roles.';
        $permissions[$viewPerm]['title'] = 'View individual group members (no roles)';
        $permissions[$updatePerm]['title'] = 'Update memberships (no roles)';
        $permissions[$deletePerm]['title'] = 'Remove memberships (no roles)';
      }
    }

    $permissions['view group_membership content']['title'] = 'View all individual group members';

    return $permissions;
  }

  /**
   * Gets roles for members of the group.
   *
   * @param \Drupal\group\Entity\GroupTypeInterface $groupType
   *   The group type.
   *
   * @return array|\Drupal\group\Entity\GroupRoleInterface[]
   *   Group roles.
   */
  protected function getGroupMemberRoles(GroupTypeInterface $groupType) {
    $roles = $groupType->getRoles(TRUE);
    $roles = array_filter($roles, function ($role) {
      /** @var \Drupal\user\RoleInterface $role */
      return $role->get('audience') == 'member';
    });
    return $roles;
  }

  /**
   * Pulls the group type from the current route.
   *
   * @return \Drupal\group\Entity\GroupTypeInterface|null
   *   The group type, or NULL if not found.
   */
  protected function getGroupTypeFromRoute() {
    // Look for the group itself.
    if ($type = $this->routeMatch->getParameter('group_type')) {
      return $type;
    }
    // @TODO: use group type of a group found in route?
    return NULL;
  }

}
